function psi = ritz(p,n,blk)
%RITZ creates a ritz collection of transfer functions. This expression is
%useful in creating time-domain IQCs.
% function psi = ritz(p,n,blk)
% ### Input arguments ###
% (1) p         -   pole/zero location in basis transfer function
% (2) n         -   highest degree of basis transfer function
% (3) blk       -   block size of transfer function
% ### Output arguments ###
% (1) psi       -   transfer function in ritz representation
% ### Ref.: Jonsson U., "Duality in Multiplier-Based Robustness Analysis",
% IEEE Transactions in Automatic Control, 44(12), pp. 2246--2256, 1999.
% ### The ritz TF has the following form,
%           [         [ jw - p ]             [ jw - p ]^n      ]^T
% psi(jw) = |eye(blk),|--------|eye(blk),...,|--------|eye(blk)|
%           [         [ jw + p ]             [ jw + p ]^n      ]
% This code guarentees a minimal realization for psi(jw).
%------------------------------------------------------------------------
R = []; % initialize to empty matrix
HH = ss(tf([1,-p],[1,p])); % basis transfer function
for k = 1:(n+1)
    if k == 1
        R = eye(blk);
    else
        nout = size(R,1);
        R = [eye(nout); zeros(blk,nout-blk), HH*eye(blk)]*R;
    end
end
psi = ss(R);
end